<?php

namespace Drupal\boutique_carousel\Plugin\Block;

use Drupal\Component\Utility\Html;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Menu\MenuTreeParameters;
use Drupal\Core\Menu\InaccessibleMenuLink;
use Drupal\Core\Render\Markup;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\file\Entity\File;
use Drupal\image\Entity\ImageStyle;
use Symfony\Component\HttpFoundation\Request;

/**
 * @Block(
 *   id = "boutique_carousel_block",
 *   admin_label = @Translation("Boutique Carousel Block"),
 * )
 */
class BoutiqueCarouselBlock extends BlockBase {

  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();
    $fids = $config['carousel_images'] ?? [];
    $group_class = 'group-order-weight';
    $form['carousel_images'] = [
      '#type' => 'managed_file',
      '#title' => new TranslatableMarkup('Upload images'),
      '#description' => new TranslatableMarkup('Best to use portrait-style images'),
      '#required' => true,
      '#multiple' => true,
      '#upload_validators' => ['file_validate_extensions' => ['png jpg jpeg gif']],
      '#upload_location' => 'public://boutique-carousel-images/',
      '#default_value' => $fids ?? [],
      '#weight' => 10,
    ];

    $order = $config['carousel_images_order'] ?? [];
    $form['carousel_images_order'] = [
      '#type' => 'table',
      '#weight' => 50,
      '#header' => [
        $this->t('Image'),
        $this->t('Header (optional)'),
        $this->t('Description (optional)'),
        $this->t('Link (optional)'),
        $this->t('Open in new tab? (optional)'),
        $this->t('Weight'),
      ],
      '#tableselect' => false,
      '#caption' => new TranslatableMarkup('Reorder images'),
      '#tabledrag' => [[
        'action' => 'order',
        'relationship' => 'sibling',
        /* hides the WEIGHT & PARENT tree columns below */
        'hidden' => TRUE,
        'group' => $group_class,
      ]],
    ];
    foreach($fids as $fid) {
      $file = File::load($fid);
      $form['carousel_images_order'][$fid]['#attributes']['class'][] = 'draggable';
      $form['carousel_images_order'][$fid]['#weight'] = $order[$fid]['weight'] ?? 0;
      $form['carousel_images_order'][$fid]['image'] = [
        '#markup' => Markup::create('<img src="' . $file->url() . '" width="100" height="100"/>'),
      ];
      $form['carousel_images_order'][$fid]['header'] = [
        '#type' => 'textfield',
        '#default_value' => $order[$fid]['header'] ?? '',
      ];
      $form['carousel_images_order'][$fid]['description'] = [
        '#type' => 'textfield',
        '#default_value' => $order[$fid]['description'] ?? '',
      ];
      $form['carousel_images_order'][$fid]['link'] = [
        '#type' => 'textfield',
        '#default_value' => $order[$fid]['link'] ?? '',
      ];
      $form['carousel_images_order'][$fid]['target'] = [
        '#type' => 'checkbox',
        '#default_value' => $order[$fid]['target'] ?? '',
      ];
      $form['carousel_images_order'][$fid]['weight'] = [
        '#type' => 'weight',
        '#title' => $this->t('Weight for @title', ['@title' => $file->getFilename()]),
        '#title_display' => 'invisible',
        '#default_value' => $order[$fid]['weight'] ?? 0,
        '#attributes' => ['class' => [$group_class]],
      ];
    }

    $styles = \Drupal::entityTypeManager()->getStorage('image_style')->loadMultiple();
    $options = ['' => new TranslatableMarkup('No style')];
    foreach($styles as $name => $style) {
      $options[$name] = $style->label();
    }
    $form['style'] = [
      '#weight' => 55,
      '#title' => new TranslatableMarkup('Select image style to apply to images'),
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => $config['style'] ?? '',
    ];
    $form['width'] = [
      '#weight' => 60,
      '#title' => new TranslatableMarkup('Image width (px)'),
      '#type' => 'textfield',
      '#default_value' => $config['width'] ?? 200,
    ];
    $form['height'] = [
      '#weight' => 65,
      '#title' => new TranslatableMarkup('Image height (px)'),
      '#type' => 'textfield',
      '#default_value' => $config['height'] ?? 400,
    ];
    return $form;
  }


  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $fids = $form_state->getValue('carousel_images');
    $this->setConfigurationValue('carousel_images', $fids);
    foreach($fids as $fid) {
      $file = File::load($fid);
      $file->setPermanent();
      $file->save();
    }
    $this->setConfigurationValue('carousel_images_order', $form_state->getValue('carousel_images_order'));
    $this->setConfigurationValue('style', $form_state->getValue('style'));
    $this->setConfigurationValue('width', $form_state->getValue('width'));
    $this->setConfigurationValue('height', $form_state->getValue('height'));
  }

  public function build() {
    $div_id = Html::getUniqueId('bcarousel');
    $config = $this->getConfiguration();
    $return['#cache'] = array(
      'contexts' => ['url'],
      'max-age' => 0,
    );
    $images = $config['carousel_images'] ?? [];
    if (empty($images)) {
      return $return;
    }
    $order = $config['carousel_images_order'] ?? [];
    uasort($images, function($a, $b) use ($order) { return $order[$a]['weight'] > $order[$b]['weight']; });
    $items = [];
    $style_name = $config['style'] ?? '';
    foreach($images as $fid) {
      $image = File::load($fid);
      if ($style_name) {
        $style = ImageStyle::load($style_name);
        $image_url = $style->buildUrl($image->getFileUri());
      }
      else {
        $image_url = $image->createFileUrl(false);
      }
      $width = $config['width'] ?? 200;
      $height = $config['height'] ?? 400;

      // TODO - refactor this nicer with renderable arrays
      $link = $order[$fid]['link'] ?? null;
      $target = $order[$fid]['target'] ?? null;
      $header = $order[$fid]['header'] ?? null;
      $description = $order[$fid]['description'] ?? null;
      $image_link = sprintf('<img src="%s" width="%d" height="%d" />', $image_url, $width, $height);
      $markup = $image_link;
      if ($header) {
        $markup .= '<h2 class="boutique-frame-title">' . $header . '</h2>';
      }
      if ($description) {
        $markup .= '<div class="boutique-frame-text">' . $description . '</div>';
      }
      if ($link) {
        if ($target) {
          $markup = '<a href="' . $link . '" target="_blank">' . $markup . '</a>';
        }
        else {
          $markup = '<a href="' . $link . '">' . $markup . '</a>';
        }
      }
      $items[$fid] = [
        '#markup' => $markup,
      ];
    }

    $return['carousel'] = [
      '#theme' => 'item_list',
      '#items' => $items,
      '#attributes' => ['id' => $div_id, 'class' => ['boutique-carousel']]
    ];
    $return['#attached']['library'][] = 'boutique_carousel/core';
    return $return;
  }
}
